//
// main.cpp
// curly
//
// Created by Bob Polis at 2020-09-01
// Copyright (c) 2020 SwiftCoder. All rights reserved.
//

#include <iostream>
#include <cstdlib>
#include <string>
#include <stdexcept>
#include <vector>
#include <cstring>
#include <getopt.h>
#include <curl/curl.h>
#include <libsclogging.hpp>

sc::logger logger {"curly", sc::loglevel::info};

void print_help() {
    std::cout << "usage: curly [-h|--version]\n";
    std::cout << "  -h, --help    show this help text and exit\n";
    std::cout << "  --version     show version number and exit\n";
}

void print_version() {
    std::cout << "curly version 1.0\n";
}

size_t write_data(void* buf, size_t sz, size_t nmemb, void* userp) {
    size_t realsize = sz * nmemb;
    SCInfo(logger, "received ", realsize, " bytes");
    std::vector<char>* dest {reinterpret_cast<std::vector<char>*>(userp)};
    size_t oldsize = dest->size();
    dest->resize(oldsize + realsize);
    std::memcpy(dest->data() + oldsize, buf, realsize);
    return realsize;
}

int main(int argc, const char * argv[]) {
    try {
        int opt_char, opt_val;
        struct option long_options[] = {
            {"help", no_argument, nullptr, 'h'},
            {"version", no_argument, &opt_val, 1},
            {nullptr, 0, nullptr, 0}
        };
        while ((opt_char = getopt_long(argc, const_cast<char* const *>(argv), "h", long_options, nullptr)) != -1) {
            std::string arg {optarg ? optarg : ""};
            switch (opt_char) {
                case 0: {
                    // handle long-only options here
                    switch (opt_val) {
                        case 1:
                            print_version();
                            return EXIT_SUCCESS;
                    }
                    break;
                }
                case 'h':
                    print_help();
                    return EXIT_SUCCESS;
                case '?':
                    throw std::runtime_error("unrecognized option");
            }
        }
        if (optind == argc) {
            // here when no file args
        }
        for (int i = optind; i < argc; ++i) {
            try {
                // process file argv[i]
            } catch (const std::runtime_error& ex) {
                std::cerr << "curly: " << ex.what() << '\n';
            }
        }
        std::cout << "hello, curly\n";

        curl_global_init(CURL_GLOBAL_ALL);
        auto easy_handle = curl_easy_init();
        curl_easy_setopt(easy_handle, CURLOPT_URL, "https://www.swiftcoder.nl/");
        curl_easy_setopt(easy_handle, CURLOPT_WRITEFUNCTION, write_data);
        std::vector<char> buf;
        curl_easy_setopt(easy_handle, CURLOPT_WRITEDATA, &buf);
        auto success = curl_easy_perform(easy_handle);
        if (success != CURLE_OK) throw std::runtime_error("could not get remote data");

        std::string text {buf.data(), buf.size()};
        std::cout << text << '\n';

        curl_easy_setopt(easy_handle, CURLOPT_URL, "https://www.swiftcoder.nl/moneydance/");
        buf.clear();
        success = curl_easy_perform(easy_handle);
        if (success != CURLE_OK) throw std::runtime_error("could not get remote data");

        std::string page2 {buf.data(), buf.size()};
        std::cout << page2 << '\n';
        
    } catch (const std::exception& ex) {
        std::cerr << "curly: " << ex.what() << '\n';
        return EXIT_FAILURE;
    }
    return EXIT_SUCCESS;
}
